# -*- coding=utf-8 -*-
from PyQt4.QtCore import QPoint, SIGNAL
from PyQt4.QtGui import QDialog, QListWidgetItem, QFont, QItemSelectionModel

from formas.dlgIdentificarVector_ui import Ui_IdentificarVector
from formas.dlgIdentificarRaster_ui import Ui_IdentificarRaster
from formas.dlgIdentificarWMS_ui import Ui_IdentificarWMS
from dialogos.utilidades import Utilidades


class DialogoIdentificarVector( QDialog, Ui_IdentificarVector ):

    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton Utilidades """
        def __call__( self, *args, **kw ) :

            if DialogoIdentificarVector.instancia is None :
                object = DialogoIdentificarVector( args[ 0 ] )
                DialogoIdentificarVector.instancia = object

            return DialogoIdentificarVector.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__( self, parent ):
        if not DialogoIdentificarVector.instancia is None:
            raise RuntimeError, 'Solo se permite una instancia de esta clase!'

        self.parent = parent
        QDialog.__init__( self, self.parent )
        self.setupUi( self )

        self.utils = Utilidades.obtenerInstancia()

        # Posicin
        self.GV = GlobalVar.obtenerInstancia() # Clase global
        if not self.GV.posicion:
            self.psize = parent.geometry()
            self.move( self.psize.width() / 20, self.psize.height() / 1.5 ) # Posicin inicial
        else:
            self.move( self.GV.posicion.x(), self.GV.posicion.y() )

        # Ajustes al tableView
        self.tblViewIdentificar.setFont( QFont( "Helvetica", 8 ) )
        self.tblViewIdentificar.horizontalHeader().setStretchLastSection( True )
        self.tblViewIdentificar.verticalHeader().setVisible( False )

        self.connect( self.lstFeatures, SIGNAL( "currentItemChanged(QListWidgetItem *,QListWidgetItem *)" ),
            self.emitirSignalNuevaSeleccion )

    def emitirSignalNuevaSeleccion( self, itemNuevo, itemPrevio ):
        """ Emite una seal ante un cambio en la seleccin """
        if not itemNuevo is None:
            self.emit( SIGNAL( "nuevaFeatureSeleccionada" ),
                self.obtenerIdFeature( itemNuevo.text() ) )

    def mostrar( self ):
        """ Muestra el dilogo partiendo de la posicin anterior """
        if self.GV.posicion:
            self.move( self.GV.posicion.x(), self.GV.posicion.y() )
            self.GV.posicion = None
        self.tblViewIdentificar.resizeRowsToContents()
        if not self.isVisible():
            self.show()

    def cerrar( self ):
        if self.isVisible():
            self.close()

    def closeEvent( self, event ):
        """ Almacena la ltima posicin del dilogo """
        self.GV.posicion = QPoint( self.x(), self.y() )

    def agregarFeature( self, idFeature ):
        """ Agrega un identificador a la lista por cada feature seleccionada """
        item = QListWidgetItem( self.lstFeatures )
        item.setText( "Id: " + str( idFeature ) )

    def seleccionarPrimeraFeature( self ):
        """ Seleccionar la primera feature en la lista """
        if self.lstFeatures.count() > 0:
            self.lstFeatures.setCurrentItem( self.lstFeatures.item( 0 ),
                QItemSelectionModel.Select )

    def limpiarFeatures( self ):
        self.lstFeatures.clear()

    def actualizarCoordenadasPunto( self, p, unidades ):
        """ Actualiza las coordenadas del punto picado """
        if unidades == 2: # Grados
            self.lblXY.setText( self.utils.formatearAGrados( p.x() ) + " | " \
                + self.utils.formatearAGrados( p.y() ) )
        else: # Unidad lineal
            self.lblXY.setText( self.utils.formatearNumero( p.x() ) + " m. | " \
                + self.utils.formatearNumero( p.y() ) + " m." )

    def obtenerIdFeature( self, text ):
        """ Extrae la subcadena que corresponde al id de la feature """
        return text[ 4: ] # La cadena original tiene la forma: "Id: 25"


class DialogoIdentificarRaster( QDialog, Ui_IdentificarRaster ):

    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton Utilidades """
        def __call__( self, *args, **kw ) :

            if DialogoIdentificarRaster.instancia is None :
                object = DialogoIdentificarRaster( args[ 0 ] )
                DialogoIdentificarRaster.instancia = object

            return DialogoIdentificarRaster.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__( self, parent ):
        if not DialogoIdentificarRaster.instancia is None:
                raise RuntimeError, 'Solo se permite una instancia de esta clase!'

        self.parent = parent
        QDialog.__init__( self, self.parent )
        self.setupUi( self )

        self.utils = Utilidades.obtenerInstancia()

        # Posicin
        self.GV = GlobalVar.obtenerInstancia() # Clase global
        if not self.GV.posicion:
            self.psize = parent.geometry()
            self.move( self.psize.width() / 20, self.psize.height() / 1.5 ) # Posicin inicial
        else:
            self.move( self.GV.posicion.x(), self.GV.posicion.y() )

        # Ajustes al tableView
        self.tblViewIdentificar.setFont( QFont( "Helvetica", 8 ) )
        self.tblViewIdentificar.horizontalHeader().setStretchLastSection( True )
        self.tblViewIdentificar.verticalHeader().setVisible( False )

    def mostrar( self ):
        """ Muestra el dilogo partiendo de la posicin anterior """
        if self.GV.posicion:
            self.move( self.GV.posicion.x(), self.GV.posicion.y() )
            self.GV.posicion = None
        self.tblViewIdentificar.resizeRowsToContents()
        if not self.isVisible():
            self.show()

    def cerrar( self ):
        if self.isVisible():
            self.close()

    def closeEvent( self, event ):
        """ Almacena la ltima posicin del dilogo """
        self.GV.posicion = QPoint( self.x(), self.y() )

    def actualizarCoordenadasPunto( self, p, unidades ):
        """ Actualiza las coordenadas del punto picado """
        if unidades == 2: # Grados
            self.lblXY.setText( self.utils.formatearAGrados( p.x() ) + " | " \
                + self.utils.formatearAGrados( p.y() ) )
        else: # Unidad lineal
            self.lblXY.setText( self.utils.formatearNumero( p.x() ) + " m. | " \
                + self.utils.formatearNumero( p.y() ) + " m." )


class DialogoIdentificarWMS( QDialog, Ui_IdentificarWMS ):

    def __init__( self, parent ):
        self.parent = parent
        QDialog.__init__( self, self.parent )
        self.setupUi( self )

        # Manejar respuestas con enlaces
        self.textBrowser.setOpenLinks( False ) # No abrir enlaces en el control
        self.connect( self.textBrowser, SIGNAL( "anchorClicked(QUrl)" ), 
            self.clickEnURL )
    
    def clickEnURL( self, url ):
        """ SLOT - Abrir cualquier enlace web en un navegador """
        import webbrowser
        webbrowser.open( url.toString() )        

    def setMessage( self, message ):
        """ Define el mensaje como texto HTML """
        self.textBrowser.setHtml( message )

    def mostrar( self ):
        """ Muestra el dilogo """        
        self.show()


class GlobalVar():
    """ Clase para manejar la variable global que guarda la Posicin del dilogo """
    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton Utilidades """
        def __call__( self, *args, **kw ) :

            if GlobalVar.instancia is None:
                object = GlobalVar()
                GlobalVar.instancia = object

            return GlobalVar.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__( self ):
        self.posicion = None