#! /usr/bin/python
# -*- coding:utf-8 -*-
import sys, os

from PyQt4.QtCore import QTranslator, QCoreApplication
from PyQt4.QtGui import QApplication, QStyleFactory, QDesktopWidget, QIcon, QMessageBox
from qgis.core import QgsApplication, QGis

from dialogos.visor import Visor

def main( argv ):
    """ Mtodo principal """
    # Crear la aplicacion Qt
    app = QApplication( argv )
    app.setStyle( QStyleFactory.create( 'Plastique' ) )
    #app.setWindowIcon( QIcon( ":/otrosIconos/imagenes/icono_igac.png" ) )
    screen = QDesktopWidget().screenGeometry() # Variable para centrar dilogos

    # Validar la versin de QGis (>= 1.1)
    if QGis.QGIS_VERSION_INT < 10100:
        msg = QMessageBox( QMessageBox.Critical, 'Error en la versin de Quantum GIS',
            'La aplicacin de Usos Alternativos de Tierras requiere una versin ' \
            + 'de Quantum GIS mayor o igual que la 1.1.0 \n\n' \
            + 'La versin que tienes instalada es: ' + QGis.QGIS_VERSION \
            + '. \nDebes instalar una versin ms reciente y volver a ejecutar el ' \
            + 'programa.' )
        msg.addButton( 'Aceptar', QMessageBox.AcceptRole )

        size =  msg.geometry()
        msg.move( ( screen.width() - size.width() )/2, ( screen.height() - size.height() )/2)
        msg.exec_()
        return

    # Inicializar QSettings para almacenar preferencias del usuario
    QCoreApplication.setOrganizationName( "IGAC" )
    QCoreApplication.setApplicationName( "UsosAlternativos" )

    # Cargar y aplicar archivos de traduccin
    translators = []
    for root, dirs, files in os.walk( "i18n" ):
        for file in files: # Filtra carpetas de traduccin
            if file.endswith( ".qm" ):
                translator = QTranslator()
                translator.load( os.path.join( "i18n", file.rstrip( ".qm" ) ) )
                translators.append( translator )
                app.installTranslator( translators[ -1 ] )

    # Fijar la ruta a las libreras de QGIS dependiendo el S.O.
    if os.name == "nt":
        osgeo4wDir = "C:/OSGeo4W/apps/qgis-1.1"
        basicDir = "C:/qgis"

        if os.path.isdir( osgeo4wDir ):
            qgis_prefix = osgeo4wDir
        else:
            qgis_prefix = basicDir
    else:
        qgis_prefix = "/usr"#"/home/tuxman/apps"

    if not os.path.isdir( qgis_prefix ):
        raise RuntimeError, "No se encuentra el directorio de QGis!!!"

    # Inicializar QGIS
    QgsApplication.setPrefixPath( qgis_prefix, True )
    QgsApplication.initQgis()

    # Crear y mostrar la ventana principal
    miVisor = Visor()
    #size =  miVisor.geometry() # OJO
    #miVisor.move( ( screen.width() - size.width() )/2, ( screen.height() - size.height() )/2)
    miVisor.show()

    # Ejecutar un loop para correr el programa
    retval = app.exec_()
    # Salir
    QgsApplication.exitQgis()
    sys.exit( retval )

if __name__ == "__main__":
    #Si el modulo es importado __name__=Visor, si es ejecutado: =__main__
    main( sys.argv )
